-- driver.adb

with LEDs;          use LEDs;
with LCD;           use LCD;
with Button;        use Button;
with Ada.Real_Time; use Ada.Real_Time;
with STM32F4;       use STM32F4;

package body Driver is

   type Index is mod 4;

   Pattern : constant array (Index) of User_LED := (Orange, Red, Blue, Green);
   --  The LEDs are not physically laid out "consecutively" in such a way that
   --  we can simply go in enumeral order to get circular rotation. Thus we
   --  define this mapping, using a consecutive index to get the physical LED
   --  blinking order desired.

   task body Controller is
      Period     : constant Time_Span := Milliseconds (500);  -- arbitrary
      Next_Start : Time := Clock;
      Next_LED   : Index := 0;
      subtype C_String is String(1..6);
      S_Blanks   : constant C_String := "      ";
      S_Orange   : constant C_String := "Orange";
      S_Red      : constant C_String := "Red   ";
      S_Blue     : constant C_String := "Blue  ";
      S_Green    : constant C_String := "Green ";
      LED_Names  : constant array(Index) of C_String
                   := (S_Orange, S_Red, S_Blue, S_Green);
      LED_X      : constant array(Index) of Byte := (14, 8, 0, 7);
      LED_Y      : constant array(Index) of Byte := (1, 2, 1, 0);

   begin
      loop
         LCD_Put_String(LED_X(Next_LED), LED_Y(Next_LED), S_Blanks);
         Off (Pattern (Next_LED));

         if Button.Current_Direction = Counterclockwise then
            Next_LED := Next_LED - 1;
         else
            Next_LED := Next_LED + 1;
         end if;

         LCD_Put_String(LED_X(Next_LED), LED_Y(Next_LED), LED_Names(Next_LED));
         On (Pattern (Next_LED));

         Next_Start := Next_Start + Period;
         delay until Next_Start;
      end loop;
   end Controller;

end Driver;
