-- leds.adb

with Ada.Unchecked_Conversion;

with Registers;     use Registers;
with STM32F4.GPIO;  use STM32F4.GPIO;
with LCD;

package body LEDs is


   function As_Word is new Ada.Unchecked_Conversion
     (Source => User_LED, Target => Word);


   procedure On (This : User_LED) is
   begin
      GPIOD.BSRR := As_Word (This);
   end On;


   procedure Off (This : User_LED) is
   begin
      GPIOD.BSRR := Shift_Left (As_Word (This), 16);
   end Off;


   All_LEDs_On  : constant Word := Green'Enum_Rep or Red'Enum_Rep or
                                   Blue'Enum_Rep  or Orange'Enum_Rep;

   pragma Compile_Time_Error
     (All_LEDs_On /= 16#F000#,
      "Invalid representation for All_LEDs_On");

   All_LEDs_Off : constant Word := Shift_Left (All_LEDs_On, 16);


   procedure All_Off is
   begin
      GPIOD.BSRR := All_LEDs_Off;
   end All_Off;


   procedure All_On is
   begin
      GPIOD.BSRR := All_LEDs_On;
   end All_On;


   procedure Initialize is
      RCC_AHB1ENR_GPIOD : constant Word := 16#08#;
      RCC_AHB1ENR_GPIOE : constant Word := 16#10#;
   begin
      --  Enable clock for GPIO-D and GPIO-E
      RCC.AHB1ENR := RCC.AHB1ENR or RCC_AHB1ENR_GPIOD or RCC_AHB1ENR_GPIOE;

      --  Configure PD12-15
      GPIOD.MODER   (12 .. 15) := (others => Mode_OUT);
      GPIOD.OTYPER  (12 .. 15) := (others => Type_PP);
      GPIOD.OSPEEDR (12 .. 15) := (others => Speed_100MHz);
      GPIOD.PUPDR   (12 .. 15) := (others => No_Pull);

      --  Configure PE4-9 (4-7 LCD data, 8 LCD RW, 9 LCD E
      GPIOE.MODER   (4 .. 9) := (others => Mode_OUT);
      GPIOE.OTYPER  (4 .. 9) := (others => Type_PP);
      GPIOE.OSPEEDR (4 .. 9) := (others => Speed_100MHz);
      GPIOE.PUPDR   (4 .. 9) := (others => No_Pull);
      --  Configure PE2
      GPIOE.MODER   (2) := Mode_OUT;
      GPIOE.OTYPER  (2) := Type_PP;
      GPIOE.OSPEEDR (2) := Speed_100MHz;
      GPIOE.PUPDR   (2) := No_Pull;
      --  Configure PE15
      GPIOE.MODER   (15) := Mode_OUT;
      GPIOE.OTYPER  (15) := Type_PP;
      GPIOE.OSPEEDR (15) := Speed_100MHz;
      GPIOE.PUPDR   (15) := No_Pull;
   end Initialize;


begin
   Initialize;
   LCD.LCD_Init;
end LEDs;
